"""
Модуль настроек библиотеки.

Словарь variable_names содержит имена переменных, 
используемых в математических выражениях.
    Ключи:
        - obj               : Переменная, обозначающая выход объекта, по умолчанию - x.
        - control           : Переменная, обозначающая вход объекта 
                              или управляющее воздействие, по умолчанию - u.
        - coefficient       : Переменная, обозначающая неизвестные 
                              коэффициенты модели, по умолчанию - a.
        - time              : Переменная, обозначающая временные такты, по умолчанию - t.
        - predicted_outputs : Переменная, обозначающая прогнозное значение 
                              выхода оюъекта, по умолчанию - xp.
        - predicted_inputs  : Переменная, обозначающая прогнозное значение входа 
                              объекта/управляющего воздействия, по умолчанию - up.
        - trajectory        : Переменная, обозначающая желаемую 
                              траекторию движения объекта, по умолчанию - xt.
        - error             : Не используется.
        - unknown_impact    : Не используется.

Словарь default_params содержит прочие параметры.
    Ключи:
        - delimiter   : Разделитель, использующийся в именах переменных, по умолчанию - '_'.
        - default_idx : Индекс переменной, по умолчанию - 0.

:Authors:
    - Vladimir Voronov
"""

from typing import Dict, Union


variable_names: Dict[str, str] = {
    'obj': 'x',
    'control': 'u',
    'coefficient': 'a',
    'time': 't',
    'predicted_outputs': 'xp',
    'predicted_inputs': 'up',
    'trajectory': 'xt',

    'error': 'e',  # ошибка
    'unknown_impact': 'h',  # неизвестное воздействие
}

# в mypy есть TypedDict
# можно так T = TypedDict('T', {'key1': str, 'key2': int})
default_params: Dict[str, Union[str, int]] = {
    'delimiter': '_',
    'default_idx': 0,
}
